package com.lee.jbuilder.syntaxchecker;

import java.awt.Container;
import java.awt.Color;
import java.util.*;
import java.text.MessageFormat;

import javax.swing.text.*;

import com.borland.primetime.editor.*;
import com.borland.primetime.node.*;
import com.borland.jbuilder.node.*;
import com.borland.jbuilder.node.java.JavaStructureIcons;

import com.lee.jbuilder.util.editor.*;

public class SyntaxChecker implements TextContextListenerFactory {
  protected static int SHOW_TOP_ERRORS_COUNT = 5;
  protected static long DELAY_ERRORS_TIME = 750;

  // EDITOR OPTIONS - Used in SyntaxChecker.properties too
  public static final String EDITOR_CATEGORY = "sc.category";
  public static final String EDITOR_OPTION_MARK_LINES = "sc.mark.lines";
  public static final String EDITOR_OPTION_UNDERLINE_TEXT = "sc.underline.text";
  public static final String EDITOR_OPTION_SHOW_TOP_ERRORS = "sc.show.top.errors";
  public static final String EDITOR_OPTION_SHOW_TOP_ERRORS_COUNT = "sc.show.top.errors.count";
  public static final String EDITOR_OPTION_AUTOEXPAND_ERROR_FOLDER = "sc.autoexpand.error.folder";
  public static final String EDITOR_OPTION_UNDERLINE_COLOR = "sc.underline.color";
  public static final String EDITOR_OPTION_DELAY_ERRORS = "sc.delay.errors";
  public static final String EDITOR_OPTION_DELAY_ERRORS_TIME = "sc.delay.errors.time";
  public static final String EDITOR_OPTION_TOOLTIPS = "sc.tooltips";

  // STATIC METHODS

  public static void initOpenTool( byte majorVersion, byte minorVersion ) {
    if ( majorVersion < 4 )
      return;

    TextContextManager.registerFactory( new SyntaxChecker() );
    ResourceBundle bundle = ResourceBundle.getBundle( SyntaxChecker.class.getName() );
    EditorOptionCategory category = new EditorOptionCategory( bundle.getString( EDITOR_CATEGORY ), JavaStructureIcons.ICON_ERROR );

    registerEditorOption( EDITOR_OPTION_MARK_LINES, category, bundle, true );
    registerEditorOption( EDITOR_OPTION_UNDERLINE_TEXT, category, bundle, true );
    registerEditorOption( EDITOR_OPTION_TOOLTIPS, category, bundle, true );
    registerEditorOption( EDITOR_OPTION_AUTOEXPAND_ERROR_FOLDER, category, bundle, false );

    SHOW_TOP_ERRORS_COUNT = Integer.parseInt( bundle.getString( EDITOR_OPTION_SHOW_TOP_ERRORS_COUNT ) );
    String display = MessageFormat.format( bundle.getString( EDITOR_OPTION_SHOW_TOP_ERRORS ), new Object[] { new Integer( SHOW_TOP_ERRORS_COUNT ) } );
    EditorManager.registerEditorOption( new BooleanEditorOption( EDITOR_OPTION_SHOW_TOP_ERRORS, category, display, false  ) );

    DELAY_ERRORS_TIME = Long.parseLong( bundle.getString( EDITOR_OPTION_DELAY_ERRORS_TIME ) );
    display = MessageFormat.format( bundle.getString( EDITOR_OPTION_DELAY_ERRORS ), new Object[] { new Long( DELAY_ERRORS_TIME ) } );
    EditorManager.registerEditorOption( new BooleanEditorOption( EDITOR_OPTION_DELAY_ERRORS, category, display, false  ) );

    // create the display style that defines the highlight color
    MasterStyleContext styleContext = EditorManager.getStyleContext();
    Style style = styleContext.addStyle( EDITOR_OPTION_UNDERLINE_COLOR, null );
    style.addAttribute( MasterStyleContext.DISPLAY_NAME, bundle.getString( EDITOR_OPTION_UNDERLINE_COLOR ) );
    StyleConstants.setBackground( style, Color.red );
  }

  // API

  public static boolean useLineMarks() {
    return EditorManager.isBooleanOptionValue( EDITOR_OPTION_MARK_LINES );
  }

  public static boolean useUnderline() {
    return EditorManager.isBooleanOptionValue( EDITOR_OPTION_UNDERLINE_TEXT );
  }

  public static boolean useToolTips() {
    return ( useUnderline() || useLineMarks() ) && EditorManager.isBooleanOptionValue( EDITOR_OPTION_TOOLTIPS );
  }

  public static int getMaxErrorCount() {
    return EditorManager.isBooleanOptionValue( EDITOR_OPTION_SHOW_TOP_ERRORS ) ? SHOW_TOP_ERRORS_COUNT : Integer.MAX_VALUE;
  }

  public static boolean autoExpandErrorFolder() {
    return EditorManager.isBooleanOptionValue( EDITOR_OPTION_AUTOEXPAND_ERROR_FOLDER );
  }

  public static long getErrorDelay() {
    return EditorManager.isBooleanOptionValue( EDITOR_OPTION_DELAY_ERRORS ) ? DELAY_ERRORS_TIME : 0;
  }

  public static Color getUnderlineColor() {
    return StyleConstants.getBackground( EditorManager.getStyleContext().getStyle( EDITOR_OPTION_UNDERLINE_COLOR ) );
  }

  // TEXTCONTEXTLISTENERFACTORY METHODS

  public boolean accepts( TextFileNode node ) {
    return node != null &&
    ( node instanceof JavaFileNode || node instanceof JspFileNode || node instanceof XMLFileNode || node instanceof HTMLFileNode );
  }

  public TextContextListener create( TextContext context ) {
    Node node = context.getNode();

    TextContextListener listener = null;
    if( node instanceof JavaFileNode )
      listener = new JavaWatcher( context );
    else if( node instanceof XMLFileNode )
      listener = new XmlWatcher( context );
    else if( node instanceof JspFileNode )
      listener = new JspWatcher( context );
    else if( node instanceof HTMLFileNode )
      listener = new HtmlWatcher( context );

    return listener;
  }

  // HELPER METHODS

  /**
   * Register the Editor option if it has not already been done.
   */
  protected static void registerEditorOption( String property, EditorOptionCategory category,  ResourceBundle bundle, boolean value ) {
    // Register EditorOption
    EditorManager.registerEditorOption( new BooleanEditorOption( property, category, bundle.getString( property ), value ) );
  }

}

