package com.lee.jbuilder.util.editor;

import java.awt.Component;
import java.awt.Container;
import java.awt.Color;
import java.awt.event.*;
import java.util.*;

import javax.swing.text.*;
import javax.swing.JTree;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.DefaultMutableTreeNode;

import com.borland.primetime.properties.*;
import com.borland.primetime.editor.*;
import com.borland.primetime.ide.*;
import com.borland.primetime.node.*;
import com.borland.primetime.viewer.*;
import com.borland.primetime.vfs.*;
import com.borland.jbuilder.jot.*;
import com.borland.jbuilder.node.*;
import com.borland.jbuilder.node.java.*;

import com.lee.jbuilder.util.*;


abstract public class HighlightContextListener implements TextContextListener {
  protected TextContext context;
  protected List highlights = new ArrayList();
  protected List instructions = new ArrayList();

  static {
    ToolTipProviderChain.getInstance().registerToolTipProvider( HighlightToolTipProvider.getInstance() );
  }

  // CONSTRUCTORS

  public HighlightContextListener( TextContext context ) {
    this.context = context;

    GutterListener listener = new GutterListener();

    // called to make tooltips works initially
    getGutter().setToolTipText( "" );
    getGutter().addMouseListener( listener );
    getGutter().addMouseMotionListener( listener );
  }

  // ABSTRACT METHODS

  abstract protected Collection getHighlightInstructions();

  // TEXTCONTEXTLISTENER METHODS

  public void treeModelChanged() {
    updateHighlighting();
  }

  // API

  protected void removeHighlighting() {
    for( Iterator i = instructions.iterator(); i.hasNext(); ) {
      HighlightInstruction instr = (HighlightInstruction) i.next();
      if( instr.getLineMark() != null )
        getDocument().removeLineMark( instr.getLineMark() );
    }

    for ( Iterator i = highlights.iterator(); i.hasNext();  ) {
      getEditor().getHighlighter().removeHighlight( i.next() );
    }

    instructions.clear();
    highlights.clear();

    // Used to Fix Bug within Gutter when clearing a LineMark
    getGutter().repaint();
  }

  protected void updateHighlighting() {
    removeHighlighting();

    TreeModel model = getModel();
    instructions.addAll( getHighlightInstructions() );

    for ( Iterator i = instructions.iterator(); i.hasNext();  ) {
      HighlightInstruction instr = ( HighlightInstruction ) i.next();

      if( instr == null )
        continue;

      if ( instr.getLineMark() != null )
        getDocument().setLineMark( instr.getLine() - 1, instr.getLineMark(), getEditor() );


      if ( instr.getHighlightPainter() != null ) {
        int p0 = instr.getStartPosition(), p1 = instr.getEndPosition();

        try {
          highlights.add( getEditor().getHighlighter().addHighlight( p0, p1, instr.getHighlightPainter() ) );
        } catch ( BadLocationException e ) { e.printStackTrace(); }
      }
    }

    getEditor().repaint();
  }

  // HELPER METHODS

  protected TextFileNode getNode() {
    return (TextFileNode) context.getNode();
  }

  protected TextNodeViewer getViewer() {
    return context.getViewer();
  }

  protected EditorPane getEditor() {
    return context.getEditor();
  }

  protected EditorDocument getDocument() {
    return context.getDocument();
  }

  protected JTree getTree() {
    return context.getTree();
  }

  protected TreeModel getModel() {
    return context.getTreeModel();
  }

  protected Gutter getGutter() {
    return context.getGutter();
  }

  protected String getToolTipForLine( int line ) {
    String toolTip = null;
    for( Iterator i = instructions.iterator(); toolTip == null && i.hasNext(); ) {
      HighlightInstruction instr = (HighlightInstruction) i.next();
      if( instr.getLineMark() != null && instr.getLine() == line )
        toolTip = instr.getToolTip();
    }

    return toolTip;
  }

  protected String getToolTipForOffset( int pos ) {
    String toolTip = null;
    for( Iterator i = instructions.iterator(); toolTip == null && i.hasNext(); ) {
      HighlightInstruction instr = (HighlightInstruction) i.next();
      if( instr.getStartPosition() <= pos && instr.getEndPosition() >= pos )
        if( instr.getHighlightPainter() != null )
          toolTip = instr.getToolTip();
    }

    return toolTip;
  }

  // INNER CLASSES

  /**
   * Listener for Mouse Events on Gutter Margin,
   * so that ToolTips can be used
   */
  protected class GutterListener implements MouseListener, MouseMotionListener {

    // MOUSELISTENER METHODS

    public void mouseClicked(MouseEvent e) {
      setToolTipText( null );
    }

    public void mousePressed(MouseEvent e) {
      setToolTipText( null );
    }

    public void mouseReleased(MouseEvent e) {
      setToolTipText( null );
    }

    public void mouseExited(MouseEvent e) {
      setToolTipText( "" );
    }

    public void mouseEntered(MouseEvent e) {
      setToolTipText( "" );
    }

   // MOUSEMOTIONLISTENER METHODS

    public void mouseDragged(MouseEvent e) {
      setToolTipText( null );
    }

    public void mouseMoved(MouseEvent e) {
      check( e );
    }

    // HELPER METHODS

    /** Set tip only if it has changed */
    void setToolTipText( String tip ) {
      String currentTip = getGutter().getToolTipText();
      if( ( tip != null && ! tip.equals( currentTip ) ) || ( tip == null && currentTip != null ) )
        getGutter().setToolTipText( tip );
    }

    void check( MouseEvent e ) {
      int pos = getEditor().viewToModel( e.getPoint() );
      int line = getEditor().getLineNumber( pos );
      setToolTipText( getToolTipForLine( line ) );
    }
  }

}