package com.lee.jbuilder.util.editor;

import java.awt.Component;
import java.awt.Container;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;

import com.borland.primetime.vfs.*;
import com.borland.primetime.viewer.TextStructureView;
import com.borland.primetime.editor.*;
import com.borland.primetime.ide.*;
import com.borland.primetime.node.*;
import com.borland.primetime.viewer.TextNodeViewer;

public class TextContext extends Context {
  protected static java.util.Timer timer = null;

  protected java.util.Timer getTimer() {
    if( timer == null ) {
      synchronized( this ) {
        if( timer == null )
          timer = new java.util.Timer( true );
      }
    }
    return timer;
  }

  /** Tells if this TextContext is initialized */
  protected volatile boolean initialized = false;

  /** key=TextContextListener value=Tasks */
  protected Map listeners = new HashMap( 5 );

  /** TreeModel, Buffer and Node Listener */
  protected TMBListener tmbListener;
  protected Buffer buffer;

  // Kept for fast retrieval
  protected transient JTree jtree = null;
  protected transient Gutter gutter = null;
  protected transient TextNodeViewer viewer = null;

  public TextContext( Browser browser, Node node ) {
    super( browser, node );
  }

  // API

  public boolean init() {
    if( initialized == false && isReady() ) {
      getTreeModel().addTreeModelListener( tmbListener = new TMBListener() );
      getNode().addNodeListener( tmbListener );

      try {
        buffer = ( (FileNode) getNode() ).getBuffer();
        buffer.addBufferListener( tmbListener );
      } catch( java.io.IOException e ) {
        throw new RuntimeException( e.getMessage() );
      }
      initialized = true;
    }

    if( initialized )
      this.treeChanged();

    return initialized;
  }

  public void close() {
    initialized = false;
    for( Iterator i = listeners.keySet().iterator(); i.hasNext(); )
      removeListener( (TextContextListener) i.next() );

    try {
      TreeModel model = getTreeModel();
      if( model != null )
        model.removeTreeModelListener( tmbListener );
    } catch( Exception e ) { e.printStackTrace(); }

    try {
      if( buffer != null )
        buffer.removeBufferListener( tmbListener );
    } catch( Exception e ) { e.printStackTrace(); }

    getNode().removeNodeListener( tmbListener );

    tmbListener = null;
    buffer = null;

    // transient data
    jtree = null;
    gutter = null;
    viewer = null;
  }

  public boolean isReady() {
    return getTreeModel() != null && getDocument() != null;
  }

  public TextNodeViewer getViewer() {
    if( viewer == null )
      viewer = ( TextNodeViewer ) getBrowser().getViewerOfType( getNode(), TextNodeViewer.class );
    return viewer;
  }

  public TextStructure getStructure() {
    TextNodeViewer viewer = getViewer();
    return viewer == null ? null : viewer.getStructure( (TextFileNode) getNode() );
  }

  /**
   * NOTE:  getViewer().getStructure( getNode() ).getTree()
   * may not return the correct JTree if the same
   * file is opened multiple times
   */
  public JTree getTree() {
    if( jtree == null ) {
      TextNodeViewer viewer = getViewer();
      if( viewer != null )
        jtree = (JTree) findComponent( viewer.getStructureComponent(), JTree.class );
    }

    return jtree;
  }

  public TreeModel getTreeModel() {
    JTree tree = getTree();
    return tree == null ? null : tree.getModel();
  }

  public EditorPane getEditor() {
    return getViewer().getEditor();
  }

  public EditorDocument getDocument() {
    EditorPane editor = getEditor();
    return editor == null ? null : (EditorDocument) getEditor().getDocument();
  }

  /**
   * Searches for a compoent of type Gutter
   */
  public Gutter getGutter() {
    if( gutter == null )
      gutter = (Gutter) findComponent( getEditor().getParent().getParent(), Gutter.class );
    return gutter;
  }

  public static Component findComponent( Container parent, Class fClass ) {
    if( fClass.isInstance( parent ) )
      return (Component) parent;

    Component[] components = parent.getComponents();
    Component rVal = null;

    for( int i = 0; rVal == null && i < components.length; i++ ) {
      if( components[i] instanceof Container )
        rVal = findComponent( (Container) components[i], fClass );
    }

    return rVal;
  }

  public void addListener( TextContextListener listener ) {
    listeners.put( listener, new Tasks( listener ) );
  }

  public void removeListener( TextContextListener listener ) {
    Tasks tasks = (Tasks) listeners.remove( listener );
    tasks.cancel();
  }

  public Collection getListeners() {
    return Collections.unmodifiableCollection( listeners.keySet() );
  }

  // HELPER METHODS

  protected void treeChanged() {
    for( Iterator i = listeners.values().iterator(); i.hasNext(); )
      ( (Tasks) i.next() ).scheduleTreeChange();
  }

  protected void sourceChanged() {
    for( Iterator i = listeners.values().iterator(); i.hasNext(); )
      ( (Tasks) i.next() ).scheduleSourceChange();
  }

  // INNERCLASSES

  class Tasks {
    TextContextListener listener;
    Task last, source, tree;

    Tasks( TextContextListener listener ) {
      this.listener = listener;
    }

    synchronized void scheduleSourceChange() {
      cancelTree();
      // If the last task was not a source change and there are no queued source tasks
      if( last == null || ( last.type != Task.SOURCE_CHANGED && source == null ) )
        schedule( source = new Task( Task.SOURCE_CHANGED, 0 ) );
    }

    synchronized void scheduleTreeChange() {
      cancelTree();
      schedule( tree = new Task( Task.TREE_CHANGED, listener.getUpdateDelay() ) );
    }

    /**
     * Schedule On Timer if delay is present
     * otherwise call SwingUtilities.invokeLater
     */
    synchronized private void schedule( Task task ) {
      if( task.delay > 0 )
        getTimer().schedule( task, task.delay );
      else
        SwingUtilities.invokeLater( task );
    }

    synchronized void cancel() {
      cancelTree();
      cancelSource();
      last = null;
    }

    synchronized void cancelTree() {
      if( tree != null ) {
        tree.cancel();
        tree = null;
      }
    }

    synchronized void cancelSource() {
      if( source != null ) {
        source.cancel();
        source = null;
      }
    }

    class Task extends TimerTask {
      static final int SOURCE_CHANGED = 1;
      static final int TREE_CHANGED = 2;
      final int type;
      long delay;

      Task( int type, long delay ) {
        this.type = type;
        this.delay = delay;
      }

      public void run() {
        if( delay > 0 )
          rescheduleForSwing();
        else
          execute();
      }

      void rescheduleForSwing() {
        this.delay = 0;
        schedule( this );
      }

      void execute() {
        if( initialized == false || isReady() == false )
          return;

        // check if tasks should be run
        synchronized( Tasks.this ) {
          if( type == SOURCE_CHANGED && this == source )
            source = null;
          else if( type == TREE_CHANGED && this == tree )
            tree = null;
          else
            return;
          last = this;
        }

        // call TextContextListener
        if( type == SOURCE_CHANGED )
          listener.sourceChanged();
        else
          listener.treeModelChanged();
      }
    }
  }


  public class TMBListener implements TreeModelListener, BufferListener, NodeListener {

    // TREEMODELLISTENER METHODS

    public void treeNodesChanged( TreeModelEvent e ) {
      treeChanged();
    }

    public void treeNodesInserted( TreeModelEvent e ) {
      treeChanged();
    }

    public void treeNodesRemoved( TreeModelEvent e ) {
      treeChanged();
    }

    public void treeStructureChanged( TreeModelEvent e ) {
      treeChanged();
    }

    // BUFFER LISTENER METHODS

    public void bufferChanged(Buffer buffer, BufferUpdater updater) {
      sourceChanged();
    }

    public void bufferLoaded(Buffer buffer) {
    }

    public void bufferSaving(Buffer buffer) {
    }

    public void bufferStateChanged(Buffer buffer, int oldState, int newState) {
      if( ( newState & Buffer.STATE_MODIFIED ) != 0 )
        sourceChanged();
    }

    // NODELISTENER

    public void nodeRenamed( Node node ) {
      Set copyListeners = new HashSet( listeners.keySet() );
      close();
      for( Iterator i = copyListeners.iterator(); i.hasNext(); )
        addListener( (TextContextListener) i.next() );

      // TreeModel is not correct until after method invocation
      SwingUtilities.invokeLater( new Runnable() {
        public void run() {
          init();
        } } );
    }
  }
}