package com.lee.jbuilder.util.editor;

import java.awt.event.MouseEvent;
import java.util.Iterator;
import java.util.List;

import com.borland.primetime.editor.EditorPane;
import com.borland.primetime.editor.ToolTipProvider;
import com.borland.primetime.ide.Browser;
import com.borland.primetime.ide.BrowserAdapter;
import com.borland.primetime.ide.NodeViewer;
import com.borland.primetime.node.Node;
import com.borland.primetime.node.TextFileNode;
import com.borland.primetime.viewer.TextNodeViewer;

/**
 * The <code>ToolTipProviderChain</code> allows for a chain of <code>ToolTipProvider</code>s
 * to respond to the <code>ToolTipProvider.getToolTipText( MouseEvent )</code> method.
 * If any registered <code>ToolTipProvider</code> gives a non-null response, then the
 * tool tip will be displayed.
 *
 * <pre>
 * EXAMPLE:
 * ToolTipProviderChain.getInstance().register( new ToolTipProvider() {
 *    // Tool Tip gives the current offset
 *    public String getToolTipText( MouseEvent evt ) {
 *      EditorPane editor = (EditorPane) evt.getSource();
 *      int pos = editor.viewToModel( e.getPoint() );
 *      return Integer.toString( pos );
 *    } } );
 * </pre>
 *
 * The ToolTipProviderChain registers the current value
 * of <code>EditorPane.tooTipProvider</code> with itself and changes
 * the protected member variable <code>EditorPane.toolTipProvider</code> to the singleton
 * instance of the ToolTipProviderChain.<p>
 *
 * NOTE:  If a ToolTipProvider has already been registered (such as the DebuggerToolTipProvider),
 * the method <code>EditorPane.registerToolTipProvider(...)</code> will
 * not register the new ToolTipProvider.
 *
 * @author Steven Lee
 */
public class ToolTipProviderChain implements ToolTipProvider {
  private static ToolTipProviderChain singleton;


  /**
   * This class is used to access the protected member variable
   * EditorPane.toolTipProvider
   */
  private static class EditorPaneHack extends EditorPane {
    public static ToolTipProvider getToolTipProvider() {
      return EditorPane.toolTipProvider;
    }

    public static void setToolTipProvider( ToolTipProvider toolTipProvider ) {
      EditorPane.toolTipProvider = toolTipProvider;
    }
  }

  /**
   * Gets the singleton instance of the ToolTipProviderChain.
   */
  public synchronized static ToolTipProviderChain getInstance() {
    if( singleton == null )
      singleton = new ToolTipProviderChain();
    return singleton;
  }

  // MEMBER VARIABLES

  private List providers = new java.util.ArrayList( 5 );

  private ToolTipProviderChain() {
    // Register Current EditorPane.toolTipProvider
    registerToolTipProvider( EditorPaneHack.getToolTipProvider() );

    // Assign protected variable
    EditorPaneHack.setToolTipProvider( this );

    createBrowserListener();
  }

  /**
   * Register a new ToolTipProvider within the chain.
   */
  public void registerToolTipProvider( ToolTipProvider provider ) {
    providers.add( provider );
  }

  /**
   * Unregisters a ToolTipProvider within the chain
   */
  public void unregisterToolTipProvider( ToolTipProvider provider ) {
    providers.remove( provider );
  }

  // TOOLTIPPROVIDER METHODS

  /**
   * Checks chain of providers for one that responds to
   * the current MouseEvent.
   */
  public String getToolTipText( MouseEvent evt ) {
    String rVal = null;

    for ( Iterator i = providers.iterator(); rVal == null && i.hasNext();  )
      rVal = ( ( ToolTipProvider ) i.next() ).getToolTipText( evt );

    return rVal;
  }

  // HELPER METHODS

  /**
   * <code>EditorPane.setText( &quot;&quot; )</code> must be called initially
   * to activate the ToolTips
   */
   private void createBrowserListener() {
    Browser.addStaticBrowserListener( new BrowserAdapter() {

      public void browserViewerActivated(Browser browser, Node node, NodeViewer nodeViewer) {
        browserNodeActivated( browser, node );
      }

      public void browserNodeActivated(Browser browser, Node node ) {
        if( node != null && node instanceof TextFileNode ) {
          TextNodeViewer viewer = ( TextNodeViewer ) browser.getViewerOfType( node, TextNodeViewer.class );
          if( viewer != null )
            viewer.getEditor().setToolTipText( "" );
        }
      }
    } );
  }

}

