package com.lee.util.swing.text;

import java.awt.*;
import java.awt.geom.*;
import java.util.Vector;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.plaf.*;

/**
 * Simple highlight painter that fills a highlighted area with
 * a solid color.
 */
public class SquigglePainter extends javax.swing.text.LayeredHighlighter.LayerPainter {
    private Color color;
    private int maxWidth;

    private final static BasicStroke DASHED_TOP = new BasicStroke( 1.0f,
                                                      BasicStroke.CAP_BUTT,
                                                      BasicStroke.JOIN_MITER,
                                                      1.0f, new float[] { 1.0f, 3.0f }, 0.0f);

    private final static BasicStroke DASHED_MIDDLE = new BasicStroke( 1.0f,
                                                      BasicStroke.CAP_BUTT,
                                                      BasicStroke.JOIN_MITER,
                                                      1.0f, new float[] { 1.0f, 1.0f }, 1.0f);

    private final static BasicStroke DASHED_BOTTOM = new BasicStroke( 1.0f,
                                                      BasicStroke.CAP_BUTT,
                                                      BasicStroke.JOIN_MITER,
                                                      1.0f, new float[] { 1.0f, 3.0f }, 3.0f);

    /**
     * Constructs a new highlight painter. If <code>c</code> is null,
     * the JTextComponent will be queried for its selection color.
     *
     * @param c the color for the highlight
     */
    public SquigglePainter( Color c ) {
      this( c, -1 );
    }

    public SquigglePainter( Color c, int maxWidth ) {
      this.color = c;
      this.maxWidth = maxWidth;
    }

    /**
     * Returns the color of the highlight.
     *
     * @return the color
     */
    public Color getColor() {
        return color;
    }

    // --- HighlightPainter methods ---------------------------------------

    /**
     * Paints a highlight.
     *
     * @param g the graphics context
     * @param offs0 the starting model offset >= 0
     * @param offs1 the ending model offset >= offs1
     * @param bounds the bounding box for the highlight
     * @param c the editor
     */
    public void paint( Graphics g, int offs0, int offs1, Shape bounds, JTextComponent c ) {
        Rectangle alloc = bounds.getBounds();

        try {
            // --- determine locations ---
            TextUI mapper = c.getUI();
            Rectangle p0 = mapper.modelToView( c, offs0 );
            Rectangle p1 = mapper.modelToView( c, offs1 );

            // --- render ---
            Color color = getColor();

            if ( color == null ) {
                g.setColor( c.getSelectionColor() );
            } else {
                g.setColor( color );
            }

            if ( p0.y == p1.y ) {
                // same line, render a rectangle
                Rectangle r = p0.union( p1 );

                // g.fillRect( r.x, r.y, r.width, r.height );
                drawLine( (Graphics2D) g, r );

            } else {
                // different lines
                int p0ToMarginWidth = alloc.x + alloc.width - p0.x;

                drawLine( (Graphics2D) g, p0 );

                /**
                 * FIX THIS
                 * g.fillRect( p0.x, p0.y, p0ToMarginWidth, p0.height );
                 *
                 * if ( ( p0.y + p0.height ) != p1.y ) {
                 * g.fillRect( alloc.x, p0.y + p0.height, alloc.width,
                 * p1.y - ( p0.y + p0.height ) );
                 * }
                 *
                 * g.fillRect( alloc.x, p1.y, ( p1.x - alloc.x ), p1.height );
                 */
            }
        } catch ( BadLocationException e ) {
            // can't render
        }
    }

    // --- LayerPainter methods ----------------------------

    /**
     * Paints a portion of a highlight.
     *
     * @param g the graphics context
     * @param offs0 the starting model offset >= 0
     * @param offs1 the ending model offset >= offs1
     * @param bounds the bounding box of the view, which is not
     * necessarily the region to paint.
     * @param c the editor
     * @param view View painting for
     * @return region drawing occured in
     */
    public Shape paintLayer( Graphics graphics, int offs0, int offs1, Shape bounds, JTextComponent c,
                             View view ) {

//       System.err.println( "paintLayer:" + offs0 + "  " + offs1 + "  start: " + view.getStartOffset() + "  end: " + view.getEndOffset() );
        Graphics2D g = (Graphics2D) graphics;
        Color color = getColor();

        if ( color == null ) {
            g.setColor( c.getSelectionColor() );
        } else {
            g.setColor( color );
        }

        if ( offs0 == view.getStartOffset() && offs1 == view.getEndOffset() ) {

            // Contained in view, can just use bounds.
            Rectangle alloc = bounds instanceof Rectangle ? ( Rectangle ) bounds : bounds.getBounds();

            drawLine( g, alloc );
            return alloc;
        } else {
            // Should only render part of View.
            try {
                // --- determine locations ---
                if( maxWidth > 0 && maxWidth < ( offs1 - offs0 ) )
                  offs1 = offs0 + maxWidth;

                Shape shape = view.modelToView( offs0, Position.Bias.Forward, offs1,
                                                Position.Bias.Backward, bounds );
                Rectangle r = ( shape instanceof Rectangle ) ? ( Rectangle ) shape
                              : shape.getBounds();

                drawLine( g, r );
                return r;
            } catch ( BadLocationException e ) {
                // can't render
            }
        }

        // Only if exception
        return null;
    }


    protected void drawLine( Graphics2D g, Rectangle r ) {
        Stroke saveStroke = g.getStroke();

        int y = r.y + r.height - 2;
        int x = r.x;

        g.setStroke( DASHED_TOP );
        g.draw( new Line2D.Double( x, y, x + r.width, y++ ) );
        g.setStroke( DASHED_MIDDLE );
        g.draw( new Line2D.Double( x, y, x + r.width, y++ ) );
        g.setStroke( DASHED_BOTTOM );
        g.draw( new Line2D.Double( x, y, x + r.width - 1, y ) );

        g.setStroke( saveStroke );
    }
}

